using System;
using System.Collections;
using System.Reflection;
using DataBroker.Dao;

namespace FirebirdDAO
{
    ///<summary>
    /// DAO Class for the Employee_project table.
    ///</summary>
    public class Employee_project : Entity
    {
        private Int16 _EMP_NO;
        private String _PROJ_ID;

        ///Default constructor
        public Employee_project()
        {

        }

        /// <summary>
        /// Constructor for joins. 
        /// Available PARENT tables to join with:
        ///    - Employee
        ///    - Project
        /// </summary>
        public Employee_project(bool Flat, params Entity[] entities)
            : this()
        {
            foreach (Entity entity in entities)
            {
                string[] entityParts = entity.GetType().ToString().Split('.');
                string entityName = entityParts[entityParts.Length - 1];
                PropertyInfo pf = GetType().GetProperty((entityName));
                if (pf != null)
                {
                    pf.SetValue(this, new EntityCollection(entity), null);
                }
                entityName = entityName + "_";
                pf = GetType().GetProperty((entityName));
                if (pf != null)
                {
                    pf.SetValue(this, entity, null);
                }
            }
            SetFlat(Flat);
        }

        ///Constructor for Joins with flat hierarchy
        public Employee_project(params Entity[] entities)
            : this(true, entities)
        {
        }

        ///<summary>
        /// Property for column Emp_no.
        ///</summary>
        [IsNotNullable]
        [IsPK]
        public Int16 Emp_no
        {
            get
            {
                return _EMP_NO;
            }
            set
            {
                _EMP_NO = value;
            }
        }
        ///<summary>
        /// Property for column Proj_id.
        ///</summary>
        [IsNotNullable]
        [IsPK]
        [MaxLength(5)]
        public String Proj_id
        {
            get
            {
                return _PROJ_ID;
            }
            set
            {
                _PROJ_ID = value;
            }
        }
        #region Employee
        private EntityCollection _Employee;
        ///<summary>
        /// Related table Employee as a collection for hierarchical retrieval.
        ///</summary>
        [NotInTable]
        [TypeContainedAttribute(typeof(Employee), "Employee")]
        public EntityCollection Employee
        {
            get
            {
                return _Employee;
            }

            set
            {
                _Employee = value;
            }
        }
        private Employee _Employee_;
        ///<summary>
        /// Related table Employee as a single object for flat retrieval.
        ///</summary>
        [NotInTable]
        public Employee Employee_
        {
            get
            {
                return _Employee_;
            }

            set
            {
                _Employee_ = value;
            }
        }
        #endregion
        #region Project
        private EntityCollection _Project;
        ///<summary>
        /// Related table Project as a collection for hierarchical retrieval.
        ///</summary>
        [NotInTable]
        [TypeContainedAttribute(typeof(Project), "Project")]
        public EntityCollection Project
        {
            get
            {
                return _Project;
            }

            set
            {
                _Project = value;
            }
        }
        private Project _Project_;
        ///<summary>
        /// Related table Project as a single object for flat retrieval.
        ///</summary>
        [NotInTable]
        public Project Project_
        {
            get
            {
                return _Project_;
            }

            set
            {
                _Project_ = value;
            }
        }
        #endregion
        /// <summary>
        /// Gets join for current entity and table.
        /// </summary>
        /// <param name="tableName">Table to join with entity.</param>
        /// <returns>Join definition for entity and table.</returns>
        public override DBJoin GetJoin(string tableName)
        {
            if (string.Compare(tableName, "Employee", true) == 0)
            {
                ArrayList left = new ArrayList();
                ArrayList right = new ArrayList();
                left.Add(tableName + ".Emp_no");
                right.Add(GetType().Name + ".Emp_no");
                return new DBJoin(left, right, JoinTypeEnumeration.INNER);
            }
            if (string.Compare(tableName, "Project", true) == 0)
            {
                ArrayList left = new ArrayList();
                ArrayList right = new ArrayList();
                left.Add(tableName + ".Proj_id");
                right.Add(GetType().Name + ".Proj_id");
                return new DBJoin(left, right, JoinTypeEnumeration.INNER);
            }
            return null;
        }
    }
}


