﻿using System;
using System.Collections;
using System.Reflection;
using DataBroker.Dao;

namespace FirebirdDAO
{
    ///<summary>
    /// DAO Class for the Salary_history table.
    ///</summary>
    public class Salary_history : Entity
    {
        private Int16 _EMP_NO;
        private DateTime _CHANGE_DATE;
        private String _UPDATER_ID;
        private Decimal _OLD_SALARY;
        private Double _PERCENT_CHANGE;
        private Double _NEW_SALARY;

        ///Default constructor
        public Salary_history()
        {

        }

        /// <summary>
        /// Constructor for joins. 
        /// Available PARENT tables to join with:
        ///    - Employee
        /// </summary>
        public Salary_history(bool Flat, params Entity[] entities)
            : this()
        {
            foreach (Entity entity in entities)
            {
                string[] entityParts = entity.GetType().ToString().Split('.');
                string entityName = entityParts[entityParts.Length - 1];
                PropertyInfo pf = GetType().GetProperty((entityName));
                if (pf != null)
                {
                    pf.SetValue(this, new EntityCollection(entity), null);
                }
                entityName = entityName + "_";
                pf = GetType().GetProperty((entityName));
                if (pf != null)
                {
                    pf.SetValue(this, entity, null);
                }
            }
            SetFlat(Flat);
        }

        ///Constructor for Joins with flat hierarchy
        public Salary_history(params Entity[] entities)
            : this(true, entities)
        {
        }

        ///<summary>
        /// Property for column Emp_no.
        ///</summary>
        [IsNotNullable]
        [IsPK]
        public Int16 Emp_no
        {
            get
            {
                return _EMP_NO;
            }
            set
            {
                _EMP_NO = value;
            }
        }
        ///<summary>
        /// Property for column Change_date.
        ///</summary>
        [IsNotNullable]
        [IsPK]
        public DateTime Change_date
        {
            get
            {
                return _CHANGE_DATE;
            }
            set
            {
                _CHANGE_DATE = value;
            }
        }
        ///<summary>
        /// Property for column Updater_id.
        ///</summary>
        [IsNotNullable]
        [IsPK]
        [MaxLength(20)]
        public String Updater_id
        {
            get
            {
                return _UPDATER_ID;
            }
            set
            {
                _UPDATER_ID = value;
            }
        }
        ///<summary>
        /// Property for column Old_salary.
        ///</summary>
        [IsNotNullable]
        public Decimal Old_salary
        {
            get
            {
                return _OLD_SALARY;
            }
            set
            {
                _OLD_SALARY = value;
            }
        }
        ///<summary>
        /// Property for column Percent_change.
        ///</summary>
        [IsNotNullable]
        public Double Percent_change
        {
            get
            {
                return _PERCENT_CHANGE;
            }
            set
            {
                _PERCENT_CHANGE = value;
            }
        }
        ///<summary>
        /// Property for column New_salary.
        ///</summary>
        public Double New_salary
        {
            get
            {
                return _NEW_SALARY;
            }
            set
            {
                _NEW_SALARY = value;
            }
        }
        #region Employee
        private EntityCollection _Employee;
        ///<summary>
        /// Related table Employee as a collection for hierarchical retrieval.
        ///</summary>
        [NotInTable]
        [TypeContainedAttribute(typeof(Employee), "Employee")]
        public EntityCollection Employee
        {
            get
            {
                return _Employee;
            }

            set
            {
                _Employee = value;
            }
        }
        private Employee _Employee_;
        ///<summary>
        /// Related table Employee as a single object for flat retrieval.
        ///</summary>
        [NotInTable]
        public Employee Employee_
        {
            get
            {
                return _Employee_;
            }

            set
            {
                _Employee_ = value;
            }
        }
        #endregion
        /// <summary>
        /// Gets join for current entity and table.
        /// </summary>
        /// <param name="tableName">Table to join with entity.</param>
        /// <returns>Join definition for entity and table.</returns>
        public override DBJoin GetJoin(string tableName)
        {
            if (string.Compare(tableName, "Employee", true) == 0)
            {
                ArrayList left = new ArrayList();
                ArrayList right = new ArrayList();
                left.Add(tableName + ".Emp_no");
                right.Add(GetType().Name + ".Emp_no");
                return new DBJoin(left, right, JoinTypeEnumeration.INNER);
            }
            return null;
        }
    }
}


