using System;
using System.Collections;
using System.Reflection;
using DataBroker.Dao;

namespace NorthwindDAO
{
	public class Employeeterritories : Entity
	{
		private Int32 _EmployeeID;
		private String _TerritoryID;
	
		///Default constructor
		public Employeeterritories ()
		{
		
		}

        /// <summary>
        /// Constructor for joins. 
		/// Available PARENT tables to join with:
		///    - Employees
		///    - Territories
        /// </summary>
        public Employeeterritories(bool Flat, params Entity[] entities)
            : this()
		{
			foreach(Entity entity in entities)
			{
				string[] entityParts = entity.GetType().ToString().Split('.');
				string entityName = entityParts[entityParts.Length - 1];
				PropertyInfo pf = GetType().GetProperty((entityName));
				if (pf != null)
				{
					pf.SetValue(this,new EntityCollection(entity), null);
				}
                entityName = entityName + "_"; ;
                pf = GetType().GetProperty((entityName));
                if (pf != null)
                {
                    pf.SetValue(this, entity, null);
                }
			}
			SetFlat(Flat);
		}
        public Employeeterritories(params Entity[] entities)
            : this(true, entities)
		{
		}
	
		[IsNotNullable]
		[IsPK]
		public Int32 Employeeid
		{
			get
			{
				return _EmployeeID;
			}
			set
			{
				_EmployeeID = value;
			}
		}
		[IsNotNullable]
		[IsPK]
		[MaxLength(20)]
		public String Territoryid
		{
			get
			{
				return _TerritoryID;
			}
			set
			{
				_TerritoryID = value;
			}
		}
        #region Employees 
        private EntityCollection _Employees;
        [NotInTable]
        [TypeContained(typeof(Employees),"Employees")]
        public EntityCollection Employees
        {
            get
            {
                return _Employees;
            }

            set
            {
                _Employees = value;
            }
        }
        private Employees _Employees_;
        [NotInTable]
        public Employees Employees_
        {
            get
            {
                return _Employees_;
            }

            set
            {
                _Employees_ = value;
            }
        }
        #endregion
        #region Territories 
        private EntityCollection _Territories;
        [NotInTable]
        [TypeContained(typeof(Territories),"Territories")]
        public EntityCollection Territories
        {
            get
            {
                return _Territories;
            }

            set
            {
                _Territories = value;
            }
        }
        private Territories _Territories_;
        [NotInTable]
        public Territories Territories_
        {
            get
            {
                return _Territories_;
            }

            set
            {
                _Territories_ = value;
            }
        }
        #endregion
        public override DBJoin GetJoin(string tableName)
        {
	        if (string.Compare(tableName, "Employees", true) == 0)
            {
                ArrayList left = new ArrayList();
                ArrayList right = new ArrayList();
				left.Add(tableName + ".Employeeid");
				right.Add(GetType().Name + ".Employeeid");
				return new DBJoin(left, right, JoinTypeEnumeration.INNER);
			}
	        if (string.Compare(tableName, "Territories", true) == 0)
            {
                ArrayList left = new ArrayList();
                ArrayList right = new ArrayList();
				left.Add(tableName + ".Territoryid");
				right.Add(GetType().Name + ".Territoryid");
				return new DBJoin(left, right, JoinTypeEnumeration.INNER);
			}
			return null;
		}
	}
}


