using System;
using System.Collections;
using System.Reflection;
using DataBroker.Dao;

namespace NorthwindDAO
{
	public class Region : Entity
	{
		private Int32 _RegionID;
		private String _RegionDescription;
	
		///Default constructor
		public Region ()
		{
		
		}

        /// <summary>
        /// Constructor for joins. 
		/// Available CHILD tables to join with:
		///    - Territories ( Territories_fk_territories_region )
        /// </summary>
        public Region(bool Flat, params Entity[] entities)
            : this()
		{
			foreach(Entity entity in entities)
			{
				string[] entityParts = entity.GetType().ToString().Split('.');
				string entityName = entityParts[entityParts.Length - 1];
				PropertyInfo pf = GetType().GetProperty((entityName));
				if (pf != null)
				{
					pf.SetValue(this,new EntityCollection(entity), null);
				}
                entityName = entityName + "_"; ;
                pf = GetType().GetProperty((entityName));
                if (pf != null)
                {
                    pf.SetValue(this, entity, null);
                }
			}
			SetFlat(Flat);
		}
        public Region(params Entity[] entities)
            : this(true, entities)
		{
		}
	
		[IsNotNullable]
		[IsPK]
		public Int32 Regionid
		{
			get
			{
				return _RegionID;
			}
			set
			{
				_RegionID = value;
			}
		}
		[IsNotNullable]
		[MaxLength(50)]
		public String Regiondescription
		{
			get
			{
				return _RegionDescription;
			}
			set
			{
				_RegionDescription = value;
			}
		}
	
        #region Territories 
        private EntityCollection _Territories;
        [NotInTable]
        [TypeContained(typeof(Territories),"Territories")]
        public EntityCollection Territories
        {
            get
            {
                return _Territories;
            }

            set
            {
                _Territories = value;
            }
        }
        private Territories _Territories_;
        [NotInTable]
        public Territories Territories_
        {
            get
            {
                return _Territories_;
            }

            set
            {
                _Territories_ = value;
            }
        }
        #endregion
        public override DBJoin GetJoin(string tableName)
        {
			if (string.Compare(tableName, "Territories", true) == 0)
			{
				ArrayList left = new ArrayList();
				ArrayList right = new ArrayList();
				left.Add(tableName + ".Regionid");
				right.Add(GetType().Name + ".Regionid");
				return new DBJoin(left, right, JoinTypeEnumeration.LEFT_OUTER);
			}
			return null;
		}
	}
}


