using System;
using System.Collections;
using System.Reflection;
using DataBroker.Dao;

namespace NorthwindDAO
{
	public class Suppliers : Entity
	{
		private Int32 _SupplierID;
		private String _CompanyName;
		private String _ContactName;
		private String _ContactTitle;
		private String _Address;
		private String _City;
		private String _Region;
		private String _PostalCode;
		private String _Country;
		private String _Phone;
		private String _Fax;
		private String _HomePage;
	
		///Default constructor
		public Suppliers ()
		{
		
		}

        /// <summary>
        /// Constructor for joins. 
		/// Available CHILD tables to join with:
		///    - Products ( Products_fk_products_suppliers )
        /// </summary>
        public Suppliers(bool Flat, params Entity[] entities)
            : this()
		{
			foreach(Entity entity in entities)
			{
				string[] entityParts = entity.GetType().ToString().Split('.');
				string entityName = entityParts[entityParts.Length - 1];
				PropertyInfo pf = GetType().GetProperty((entityName));
				if (pf != null)
				{
					pf.SetValue(this,new EntityCollection(entity), null);
				}
                entityName = entityName + "_"; ;
                pf = GetType().GetProperty((entityName));
                if (pf != null)
                {
                    pf.SetValue(this, entity, null);
                }
			}
			SetFlat(Flat);
		}
        public Suppliers(params Entity[] entities)
            : this(true, entities)
		{
		}
	
		[IsNotNullable]
		[IsPK]
		[IsIdentity]
		[SequenceName("Suppliers")]
		public Int32 Supplierid
		{
			get
			{
				return _SupplierID;
			}
		}
		[IsNotNullable]
		[MaxLength(40)]
		public String Companyname
		{
			get
			{
				return _CompanyName;
			}
			set
			{
				_CompanyName = value;
			}
		}
		[MaxLength(30)]
		public String Contactname
		{
			get
			{
				return _ContactName;
			}
			set
			{
				_ContactName = value;
			}
		}
		[MaxLength(30)]
		public String Contacttitle
		{
			get
			{
				return _ContactTitle;
			}
			set
			{
				_ContactTitle = value;
			}
		}
		[MaxLength(60)]
		public String Address
		{
			get
			{
				return _Address;
			}
			set
			{
				_Address = value;
			}
		}
		[MaxLength(15)]
		public String City
		{
			get
			{
				return _City;
			}
			set
			{
				_City = value;
			}
		}
		[MaxLength(15)]
		public String Region
		{
			get
			{
				return _Region;
			}
			set
			{
				_Region = value;
			}
		}
		[MaxLength(10)]
		public String Postalcode
		{
			get
			{
				return _PostalCode;
			}
			set
			{
				_PostalCode = value;
			}
		}
		[MaxLength(15)]
		public String Country
		{
			get
			{
				return _Country;
			}
			set
			{
				_Country = value;
			}
		}
		[MaxLength(24)]
		public String Phone
		{
			get
			{
				return _Phone;
			}
			set
			{
				_Phone = value;
			}
		}
		[MaxLength(24)]
		public String Fax
		{
			get
			{
				return _Fax;
			}
			set
			{
				_Fax = value;
			}
		}
		[MaxLength(16)]
		public String Homepage
		{
			get
			{
				return _HomePage;
			}
			set
			{
				_HomePage = value;
			}
		}
	
        #region Products 
        private EntityCollection _Products;
        [NotInTable]
        [TypeContained(typeof(Products),"Products")]
        public EntityCollection Products
        {
            get
            {
                return _Products;
            }

            set
            {
                _Products = value;
            }
        }
        private Products _Products_;
        [NotInTable]
        public Products Products_
        {
            get
            {
                return _Products_;
            }

            set
            {
                _Products_ = value;
            }
        }
        #endregion
        public override DBJoin GetJoin(string tableName)
        {
			if (string.Compare(tableName, "Products", true) == 0)
			{
				ArrayList left = new ArrayList();
				ArrayList right = new ArrayList();
				left.Add(tableName + ".Supplierid");
				right.Add(GetType().Name + ".Supplierid");
				return new DBJoin(left, right, JoinTypeEnumeration.LEFT_OUTER);
			}
			return null;
		}
	}
}


