using System;
using System.Collections;
using System.Reflection;
using DataBroker.Dao;

namespace OraHRDAO
{
	///<summary>
	/// DAO Class for the COUNTRIES table.
	///</summary>
	public class COUNTRIES : Entity
	{
		private String _COUNTRY_ID;
		private String _COUNTRY_NAME;
		private Int32 _REGION_ID;
	
		///Default constructor
		public COUNTRIES ()
		{
		
		}

        /// <summary>
        /// Constructor for joins. 
		/// Available CHILD tables to join with:
		///    - LOCATIONS ( LOCATIONS_LOC_C_ID_FK )
		/// Available PARENT tables to join with:
		///    - REGIONS
        /// </summary>
        public COUNTRIES(bool Flat, params Entity[] entities)
            : this()
		{
			foreach(Entity entity in entities)
			{
				string[] entityParts = entity.GetType().ToString().Split('.');
				string entityName = entityParts[entityParts.Length - 1];
				PropertyInfo pf = GetType().GetProperty((entityName));
				if (pf != null)
				{
					pf.SetValue(this,new EntityCollection(entity), null);
				}
                entityName = entityName + "_";
                pf = GetType().GetProperty((entityName));
                if (pf != null)
                {
                    pf.SetValue(this, entity, null);
                }
			}
			SetFlat(Flat);
		}

		///Constructor for Joins with flat hierarchy
        public COUNTRIES(params Entity[] entities)
            : this(true, entities)
		{
		}
	
		///<summary>
		/// Property for column COUNTRY_ID.
		///</summary>
		[IsNotNullable]
		[IsPK]
		[MaxLength(2)]
		public String COUNTRY_ID
		{
			get
			{
				return _COUNTRY_ID;
			}
			set
			{
				_COUNTRY_ID = value;
			}
		}
		///<summary>
		/// Property for column COUNTRY_NAME.
		///</summary>
		[MaxLength(40)]
		public String COUNTRY_NAME
		{
			get
			{
				return _COUNTRY_NAME;
			}
			set
			{
				_COUNTRY_NAME = value;
			}
		}
		///<summary>
		/// Property for column REGION_ID.
		///</summary>
		public Int32 REGION_ID
		{
			get
			{
				return _REGION_ID;
			}
			set
			{
				_REGION_ID = value;
			}
		}
	
        #region LOCATIONS 
        private EntityCollection _LOCATIONS;
		///<summary>
		/// Related table LOCATIONS as a collection for hierarchical retrieval.
		///</summary>
        [NotInTable]
        [TypeContainedAttribute(typeof(LOCATIONS),"LOCATIONS")]
        public EntityCollection LOCATIONS
        {
            get
            {
                return _LOCATIONS;
            }

            set
            {
                _LOCATIONS = value;
            }
        }
        private LOCATIONS _LOCATIONS_;
		///<summary>
		/// Related table LOCATIONS as a single object for flat retrieval.
		///</summary>
        [NotInTable]
        public LOCATIONS LOCATIONS_
        {
            get
            {
                return _LOCATIONS_;
            }

            set
            {
                _LOCATIONS_ = value;
            }
        }
        #endregion
        #region REGIONS 
        private EntityCollection _REGIONS;
		///<summary>
		/// Related table REGIONS as a collection for hierarchical retrieval.
		///</summary>
        [NotInTable]
        [TypeContainedAttribute(typeof(REGIONS),"REGIONS")]
        public EntityCollection REGIONS
        {
            get
            {
                return _REGIONS;
            }

            set
            {
                _REGIONS = value;
            }
        }
        private REGIONS _REGIONS_;
		///<summary>
		/// Related table REGIONS as a single object for flat retrieval.
		///</summary>
        [NotInTable]
        public REGIONS REGIONS_
        {
            get
            {
                return _REGIONS_;
            }

            set
            {
                _REGIONS_ = value;
            }
        }
        #endregion
	    /// <summary>
	    /// Gets join for current entity and table.
	    /// </summary>
	    /// <param name="tableName">Table to join with entity.</param>
	    /// <returns>Join definition for entity and table.</returns>
        public override DBJoin GetJoin(string tableName)
        {
			if (string.Compare(tableName, "LOCATIONS", StringComparison.OrdinalIgnoreCase) == 0)
			{
				ArrayList left = new ArrayList();
				ArrayList right = new ArrayList();
				left.Add(tableName + ".COUNTRY_ID");
				right.Add(GetType().Name + ".COUNTRY_ID");
				return new DBJoin(left, right, JoinTypeEnumeration.LEFT_OUTER);
			}
	        if (string.Compare(tableName, "REGIONS", true) == 0)
            {
                ArrayList left = new ArrayList();
                ArrayList right = new ArrayList();
				left.Add(tableName + ".REGION_ID");
				right.Add(GetType().Name + ".REGION_ID");
				return new DBJoin(left, right, JoinTypeEnumeration.LEFT_OUTER);
			}
			return null;
		}
	}
}


