using System;
using System.Collections;
using System.Reflection;
using DataBroker.Dao;

namespace OraHRDAO
{
	///<summary>
	/// DAO Class for the JOB_HISTORY table.
	///</summary>
	public class JOB_HISTORY : Entity
	{
		private Int32 _EMPLOYEE_ID;
		private DateTime _START_DATE;
		private DateTime _END_DATE;
		private String _JOB_ID;
		private Int32 _DEPARTMENT_ID;
	
		///Default constructor
		public JOB_HISTORY ()
		{
		
		}

        /// <summary>
        /// Constructor for joins. 
		/// Available PARENT tables to join with:
		///    - DEPARTMENTS
		///    - EMPLOYEES
		///    - JOBS
        /// </summary>
        public JOB_HISTORY(bool Flat, params Entity[] entities)
            : this()
		{
			foreach(Entity entity in entities)
			{
				string[] entityParts = entity.GetType().ToString().Split('.');
				string entityName = entityParts[entityParts.Length - 1];
				PropertyInfo pf = GetType().GetProperty((entityName));
				if (pf != null)
				{
					pf.SetValue(this,new EntityCollection(entity), null);
				}
                entityName = entityName + "_";
                pf = GetType().GetProperty((entityName));
                if (pf != null)
                {
                    pf.SetValue(this, entity, null);
                }
			}
			SetFlat(Flat);
		}

		///Constructor for Joins with flat hierarchy
        public JOB_HISTORY(params Entity[] entities)
            : this(true, entities)
		{
		}
	
		///<summary>
		/// Property for column EMPLOYEE_ID.
		///</summary>
		[IsNotNullable]
		[IsPK]
		public Int32 EMPLOYEE_ID
		{
			get
			{
				return _EMPLOYEE_ID;
			}
			set
			{
				_EMPLOYEE_ID = value;
			}
		}
		///<summary>
		/// Property for column START_DATE.
		///</summary>
		[IsNotNullable]
		[IsPK]
		public DateTime START_DATE
		{
			get
			{
				return _START_DATE;
			}
			set
			{
				_START_DATE = value;
			}
		}
		///<summary>
		/// Property for column END_DATE.
		///</summary>
		[IsNotNullable]
		public DateTime END_DATE
		{
			get
			{
				return _END_DATE;
			}
			set
			{
				_END_DATE = value;
			}
		}
		///<summary>
		/// Property for column JOB_ID.
		///</summary>
		[IsNotNullable]
		[MaxLength(10)]
		public String JOB_ID
		{
			get
			{
				return _JOB_ID;
			}
			set
			{
				_JOB_ID = value;
			}
		}
		///<summary>
		/// Property for column DEPARTMENT_ID.
		///</summary>
		public Int32 DEPARTMENT_ID
		{
			get
			{
				return _DEPARTMENT_ID;
			}
			set
			{
				_DEPARTMENT_ID = value;
			}
		}
        #region DEPARTMENTS 
        private EntityCollection _DEPARTMENTS;
		///<summary>
		/// Related table DEPARTMENTS as a collection for hierarchical retrieval.
		///</summary>
        [NotInTable]
        [TypeContainedAttribute(typeof(DEPARTMENTS),"DEPARTMENTS")]
        public EntityCollection DEPARTMENTS
        {
            get
            {
                return _DEPARTMENTS;
            }

            set
            {
                _DEPARTMENTS = value;
            }
        }
        private DEPARTMENTS _DEPARTMENTS_;
		///<summary>
		/// Related table DEPARTMENTS as a single object for flat retrieval.
		///</summary>
        [NotInTable]
        public DEPARTMENTS DEPARTMENTS_
        {
            get
            {
                return _DEPARTMENTS_;
            }

            set
            {
                _DEPARTMENTS_ = value;
            }
        }
        #endregion
        #region EMPLOYEES 
        private EntityCollection _EMPLOYEES;
		///<summary>
		/// Related table EMPLOYEES as a collection for hierarchical retrieval.
		///</summary>
        [NotInTable]
        [TypeContainedAttribute(typeof(EMPLOYEES),"EMPLOYEES")]
        public EntityCollection EMPLOYEES
        {
            get
            {
                return _EMPLOYEES;
            }

            set
            {
                _EMPLOYEES = value;
            }
        }
        private EMPLOYEES _EMPLOYEES_;
		///<summary>
		/// Related table EMPLOYEES as a single object for flat retrieval.
		///</summary>
        [NotInTable]
        public EMPLOYEES EMPLOYEES_
        {
            get
            {
                return _EMPLOYEES_;
            }

            set
            {
                _EMPLOYEES_ = value;
            }
        }
        #endregion
        #region JOBS 
        private EntityCollection _JOBS;
		///<summary>
		/// Related table JOBS as a collection for hierarchical retrieval.
		///</summary>
        [NotInTable]
        [TypeContainedAttribute(typeof(JOBS),"JOBS")]
        public EntityCollection JOBS
        {
            get
            {
                return _JOBS;
            }

            set
            {
                _JOBS = value;
            }
        }
        private JOBS _JOBS_;
		///<summary>
		/// Related table JOBS as a single object for flat retrieval.
		///</summary>
        [NotInTable]
        public JOBS JOBS_
        {
            get
            {
                return _JOBS_;
            }

            set
            {
                _JOBS_ = value;
            }
        }
        #endregion
	    /// <summary>
	    /// Gets join for current entity and table.
	    /// </summary>
	    /// <param name="tableName">Table to join with entity.</param>
	    /// <returns>Join definition for entity and table.</returns>
        public override DBJoin GetJoin(string tableName)
        {
	        if (string.Compare(tableName, "DEPARTMENTS", true) == 0)
            {
                ArrayList left = new ArrayList();
                ArrayList right = new ArrayList();
				left.Add(tableName + ".DEPARTMENT_ID");
				right.Add(GetType().Name + ".DEPARTMENT_ID");
				return new DBJoin(left, right, JoinTypeEnumeration.LEFT_OUTER);
			}
	        if (string.Compare(tableName, "EMPLOYEES", true) == 0)
            {
                ArrayList left = new ArrayList();
                ArrayList right = new ArrayList();
				left.Add(tableName + ".EMPLOYEE_ID");
				right.Add(GetType().Name + ".EMPLOYEE_ID");
				return new DBJoin(left, right, JoinTypeEnumeration.INNER);
			}
	        if (string.Compare(tableName, "JOBS", true) == 0)
            {
                ArrayList left = new ArrayList();
                ArrayList right = new ArrayList();
				left.Add(tableName + ".JOB_ID");
				right.Add(GetType().Name + ".JOB_ID");
				return new DBJoin(left, right, JoinTypeEnumeration.INNER);
			}
			return null;
		}
	}
}


