using System;
using System.Collections;
using System.Reflection;
using DataBroker.Dao;

namespace OraHRDAO
{
	///<summary>
	/// DAO Class for the LOCATIONS table.
	///</summary>
	public class LOCATIONS : Entity
	{
		private Int32 _LOCATION_ID;
		private String _STREET_ADDRESS;
		private String _POSTAL_CODE;
		private String _CITY;
		private String _STATE_PROVINCE;
		private String _COUNTRY_ID;
	
		///Default constructor
		public LOCATIONS ()
		{
		
		}

        /// <summary>
        /// Constructor for joins. 
		/// Available CHILD tables to join with:
		///    - DEPARTMENTS ( DEPARTMENTS_DEPT_LOC_FK )
		/// Available PARENT tables to join with:
		///    - COUNTRIES
        /// </summary>
        public LOCATIONS(bool Flat, params Entity[] entities)
            : this()
		{
			foreach(Entity entity in entities)
			{
				string[] entityParts = entity.GetType().ToString().Split('.');
				string entityName = entityParts[entityParts.Length - 1];
				PropertyInfo pf = GetType().GetProperty((entityName));
				if (pf != null)
				{
					pf.SetValue(this,new EntityCollection(entity), null);
				}
                entityName = entityName + "_";
                pf = GetType().GetProperty((entityName));
                if (pf != null)
                {
                    pf.SetValue(this, entity, null);
                }
			}
			SetFlat(Flat);
		}

		///Constructor for Joins with flat hierarchy
        public LOCATIONS(params Entity[] entities)
            : this(true, entities)
		{
		}
	
		///<summary>
		/// Property for column LOCATION_ID.
		///</summary>
		[IsNotNullable]
		[IsPK]
		public Int32 LOCATION_ID
		{
			get
			{
				return _LOCATION_ID;
			}
			set
			{
				_LOCATION_ID = value;
			}
		}
		///<summary>
		/// Property for column STREET_ADDRESS.
		///</summary>
		[MaxLength(40)]
		public String STREET_ADDRESS
		{
			get
			{
				return _STREET_ADDRESS;
			}
			set
			{
				_STREET_ADDRESS = value;
			}
		}
		///<summary>
		/// Property for column POSTAL_CODE.
		///</summary>
		[MaxLength(12)]
		public String POSTAL_CODE
		{
			get
			{
				return _POSTAL_CODE;
			}
			set
			{
				_POSTAL_CODE = value;
			}
		}
		///<summary>
		/// Property for column CITY.
		///</summary>
		[IsNotNullable]
		[MaxLength(30)]
		public String CITY
		{
			get
			{
				return _CITY;
			}
			set
			{
				_CITY = value;
			}
		}
		///<summary>
		/// Property for column STATE_PROVINCE.
		///</summary>
		[MaxLength(25)]
		public String STATE_PROVINCE
		{
			get
			{
				return _STATE_PROVINCE;
			}
			set
			{
				_STATE_PROVINCE = value;
			}
		}
		///<summary>
		/// Property for column COUNTRY_ID.
		///</summary>
		[MaxLength(2)]
		public String COUNTRY_ID
		{
			get
			{
				return _COUNTRY_ID;
			}
			set
			{
				_COUNTRY_ID = value;
			}
		}
	
        #region DEPARTMENTS 
        private EntityCollection _DEPARTMENTS;
		///<summary>
		/// Related table DEPARTMENTS as a collection for hierarchical retrieval.
		///</summary>
        [NotInTable]
        [TypeContainedAttribute(typeof(DEPARTMENTS),"DEPARTMENTS")]
        public EntityCollection DEPARTMENTS
        {
            get
            {
                return _DEPARTMENTS;
            }

            set
            {
                _DEPARTMENTS = value;
            }
        }
        private DEPARTMENTS _DEPARTMENTS_;
		///<summary>
		/// Related table DEPARTMENTS as a single object for flat retrieval.
		///</summary>
        [NotInTable]
        public DEPARTMENTS DEPARTMENTS_
        {
            get
            {
                return _DEPARTMENTS_;
            }

            set
            {
                _DEPARTMENTS_ = value;
            }
        }
        #endregion
        #region COUNTRIES 
        private EntityCollection _COUNTRIES;
		///<summary>
		/// Related table COUNTRIES as a collection for hierarchical retrieval.
		///</summary>
        [NotInTable]
        [TypeContainedAttribute(typeof(COUNTRIES),"COUNTRIES")]
        public EntityCollection COUNTRIES
        {
            get
            {
                return _COUNTRIES;
            }

            set
            {
                _COUNTRIES = value;
            }
        }
        private COUNTRIES _COUNTRIES_;
		///<summary>
		/// Related table COUNTRIES as a single object for flat retrieval.
		///</summary>
        [NotInTable]
        public COUNTRIES COUNTRIES_
        {
            get
            {
                return _COUNTRIES_;
            }

            set
            {
                _COUNTRIES_ = value;
            }
        }
        #endregion
	    /// <summary>
	    /// Gets join for current entity and table.
	    /// </summary>
	    /// <param name="tableName">Table to join with entity.</param>
	    /// <returns>Join definition for entity and table.</returns>
        public override DBJoin GetJoin(string tableName)
        {
			if (string.Compare(tableName, "DEPARTMENTS", StringComparison.OrdinalIgnoreCase) == 0)
			{
				ArrayList left = new ArrayList();
				ArrayList right = new ArrayList();
				left.Add(tableName + ".LOCATION_ID");
				right.Add(GetType().Name + ".LOCATION_ID");
				return new DBJoin(left, right, JoinTypeEnumeration.LEFT_OUTER);
			}
	        if (string.Compare(tableName, "COUNTRIES", true) == 0)
            {
                ArrayList left = new ArrayList();
                ArrayList right = new ArrayList();
				left.Add(tableName + ".COUNTRY_ID");
				right.Add(GetType().Name + ".COUNTRY_ID");
				return new DBJoin(left, right, JoinTypeEnumeration.LEFT_OUTER);
			}
			return null;
		}
	}
}


