using System;
using System.Collections;
using System.Reflection;
using DataBroker.Dao;

namespace ScottDAO
{
    ///<summary>
    /// DAO Class for the EMP table.
    ///</summary>
    public class EMP : Entity
    {
        private Int32 _EMPNO;
        private String _ENAME;
        private String _JOB;
        private Int32 _MGR;
        private DateTime _HIREDATE;
        private Single _SAL;
        private Single _COMM;
        private Int32 _DEPTNO;
	
        ///Default constructor
        public EMP ()
        {
		
        }

        /// <summary>
        /// Constructor for joins. 
        /// Available PARENT tables to join with:
        ///    - DEPT
        /// </summary>
        public EMP(bool Flat, params Entity[] entities)
            : this()
        {
            foreach(Entity entity in entities)
            {
                string[] entityParts = entity.GetType().ToString().Split('.');
                string entityName = entityParts[entityParts.Length - 1];
                PropertyInfo pf = GetType().GetProperty((entityName));
                if (pf != null)
                {
                    pf.SetValue(this,new EntityCollection(entity), null);
                }
                entityName = entityName + "_";
                pf = GetType().GetProperty((entityName));
                if (pf != null)
                {
                    pf.SetValue(this, entity, null);
                }
            }
            SetFlat(Flat);
        }

        ///Constructor for Joins with flat hierarchy
        public EMP(params Entity[] entities)
            : this(true, entities)
        {
        }
	
        ///<summary>
        /// Property for column EMPNO.
        ///</summary>
        [IsNotNullable]
        [IsPK]
        public Int32 EMPNO
        {
            get
            {
                return _EMPNO;
            }
            set
            {
                _EMPNO = value;
            }
        }
        ///<summary>
        /// Property for column ENAME.
        ///</summary>
        [MaxLength(10)]
        public String ENAME
        {
            get
            {
                return _ENAME;
            }
            set
            {
                _ENAME = value;
            }
        }
        ///<summary>
        /// Property for column JOB.
        ///</summary>
        [MaxLength(9)]
        public String JOB
        {
            get
            {
                return _JOB;
            }
            set
            {
                _JOB = value;
            }
        }
        ///<summary>
        /// Property for column MGR.
        ///</summary>
        public Int32 MGR
        {
            get
            {
                return _MGR;
            }
            set
            {
                _MGR = value;
            }
        }
        ///<summary>
        /// Property for column HIREDATE.
        ///</summary>
        public DateTime HIREDATE
        {
            get
            {
                return _HIREDATE;
            }
            set
            {
                _HIREDATE = value;
            }
        }
        ///<summary>
        /// Property for column SAL.
        ///</summary>
        public Single SAL
        {
            get
            {
                return _SAL;
            }
            set
            {
                _SAL = value;
            }
        }
        ///<summary>
        /// Property for column COMM.
        ///</summary>
        public Single COMM
        {
            get
            {
                return _COMM;
            }
            set
            {
                _COMM = value;
            }
        }
        ///<summary>
        /// Property for column DEPTNO.
        ///</summary>
        public Int32 DEPTNO
        {
            get
            {
                return _DEPTNO;
            }
            set
            {
                _DEPTNO = value;
            }
        }
        #region DEPT 
        private EntityCollection _DEPT;
        ///<summary>
        /// Related table DEPT as a collection for hierarchical retrieval.
        ///</summary>
        [NotInTable]
        [TypeContainedAttribute(typeof(DEPT),"DEPT")]
        public EntityCollection DEPT
        {
            get
            {
                return _DEPT;
            }

            set
            {
                _DEPT = value;
            }
        }
        private DEPT _DEPT_;
        ///<summary>
        /// Related table DEPT as a single object for flat retrieval.
        ///</summary>
        [NotInTable]
        public DEPT DEPT_
        {
            get
            {
                return _DEPT_;
            }

            set
            {
                _DEPT_ = value;
            }
        }
        #endregion
        /// <summary>
        /// Gets join for current entity and table.
        /// </summary>
        /// <param name="tableName">Table to join with entity.</param>
        /// <returns>Join definition for entity and table.</returns>
        public override DBJoin GetJoin(string tableName)
        {
            if (string.Compare(tableName, "DEPT", true) == 0)
            {
                ArrayList left = new ArrayList();
                ArrayList right = new ArrayList();
                left.Add(tableName + ".DEPTNO");
                right.Add(GetType().Name + ".DEPTNO");
                return new DBJoin(left, right, JoinTypeEnumeration.LEFT_OUTER);
            }
            return null;
        }
    }
}